@@ -4,12 +4,12 @@ import gtk
 import gobject
 import random
 
-VERSION = "0.3.8"
+VERSION = "0.4.0"
 IS_MAEMO = commands.getoutput("uname -m")[0:3] == "arm" # if cpu type is ARM, we are on maemo
 
 # general settings
 db_file = os.path.expanduser("~/.musicdb")
-known_exts = ("mp3", "ogg")
+known_exts = ("mp3", "ogg", "mp4", "wma")
 
 # device specific settings
 if IS_MAEMO:
