@@ -1,6 +1,9 @@
 import gtk
 import hildon
 import gobject
+import time
+import urllib2
+import sys
 
 from youamp.ui.window import Window
 from youamp.ui.about import AboutDialog
@@ -11,6 +14,7 @@ from youamp import Playlist, IS_MAEMO, VERSION
 from youamp.library import Library, check_db
 from youamp.player import Player
 from youamp.config import Config
+import youamp.scrobbler as scrobbler
 
 if IS_MAEMO:
     import osso
@@ -25,12 +29,24 @@ class Controller:
         self._player = Player(self._config)
         self._library = Library()
         self._window = Window(self._player, self._config, self._library)
+        self._lastfm_connected = False
+        self._banner = None
         
         # programm
         program = hildon.Program()
         program.add_window(self._window)
         program.set_property("can-hibernate", True)
-        
+
+        gobject.set_application_name("YouAmp")
+   
+        # last.fm submission
+        if self._config["lastfm-user"] != "":
+            try:    
+                scrobbler.login(self._config["lastfm-user"], self._config["lastfm-pass"], client=("you", VERSION))
+                self._lastfm_connected = True
+            except (urllib2.URLError, scrobbler.AuthError), e:
+                sys.stderr.write("lastfm.login: %s\n" % e)
+            
         # RPC
         if IS_MAEMO:
             osso_rpc = osso.Rpc(osso_c)
@@ -41,9 +57,7 @@ class Controller:
         # dialogs
         self._about_dialog = None
         self._preferences = None
-        
-        gobject.set_application_name("YouAmp")
-            
+                    
         # menu
         menu = gtk.Menu()
         self._window.set_menu(menu)
@@ -90,32 +104,34 @@ class Controller:
         # restore config
         if self._player.tracks() > 0:
             self._player.load_track()
-            
-        if self._config["is-browser"]:
-            browse.set_active(True)
-        else:
-            search.set_active(True)
         
         if not self._config["shuffle"]:
             self._order_menu[self._config["order-by"]].set_active(True)
 
-        # window signals
+        # signals
         self._window.connect("destroy", self.quit)
-        self._window.connect("key-press-event", self._on_key_press)
+        self._window.connect("key-press-event", self._handle_keypress)
+        
         self._player.connect("song-played", self._on_song_played)
+        self._config.notify_add("music-folder", self._refresh_library)
+        
+        if self._lastfm_connected:
+            self._player.connect("song-changed", self._now_playing)
 
         # order signals
         self._order_menu["artist"].connect("activate", lambda caller: self._set_list_order("artist"))
         self._order_menu["date"].connect("activate", lambda caller: self._set_list_order("date"))
         self._order_menu["playcount"].connect("activate", lambda caller: self._set_list_order("playcount"))
-        
-        # banner
-        self._banner = hildon.hildon_banner_show_animation(self._window, "qgn_indi_pball_a", _("Updating database"))
-        self._library.connect("update-complete", self._destroy_banner)
+
+        self._library.connect("update-complete", self._on_library_updated)
 
         # start updating library
-        self._library.update(self._config["music-folder"])
+        self._refresh_library()
     
+    def _refresh_library(self, *args):
+        self._banner = hildon.hildon_banner_show_animation(self._window, "qgn_indi_pball_a", _("Updating database"))
+        self._library.update(self._config["music-folder"])
+        
     def _handle_rpc(self, interface, method, arguments, user_data):
         action = dict();
         action["toggle"] = self._window._controls.on_toggle
@@ -143,20 +159,49 @@ class Controller:
         
         # music-folder was added in v0.3.8
         if not self._config.has_key("music-folder"):
-            self._config["music-folder"] = "/media/mmc2"
+            self._config["music-folder"] = "/home/user"
+            
+        # last.fm support was added in v0.4.0
+        if not self._config.has_key("lastfm-user"):
+            self._config["lastfm-user"] = ""
+            self._config["lastfm-pass"] = ""
 
     def _set_list_order(self, order):
         self._config["order-by"] = order
         self._config["shuffle"] = False
         self._config["pos"] = 0
+        
+        self._refresh_playlist()
 
+    def _refresh_playlist(self):
         playlist = self._library.get_tracks(self._config)
             
         self._player.playlist.update(playlist)
 
-    def _on_song_played(self, caller, song):
+    def _on_song_played(self, player, song):
+        played = player.get_time()
+        
+        # workaround for bugged meamo
+        if not song.has_key("duration"):
+            return
+        
+        if played < song["duration"]/2 or played < 240 or song["duration"] < 30:
+            return
+        
         self._library.increment_played(song.uri)
+        
+        if self._lastfm_connected:
+            try:
+                scrobbler.submit(song["artist"], song["title"], int(time.time()), album=song["album"], length=song["duration"])
+            except urllib2.URLError, e:
+                sys.stderr.write("lastfm.submit: %s\n" % e)
 
+    def _now_playing(self, caller, song):
+        try:
+            scrobbler.now_playing(song["artist"], song["title"], song["album"])
+        except urllib2.URLError, e:
+            sys.stderr.write("lastfm.now_playing: %s\n" % e)
+        
     def _show_about_dialog(self, caller):
         if self._about_dialog is None:
             self._about_dialog = AboutDialog()
@@ -165,15 +210,18 @@ class Controller:
     
     def _show_preferences(self, caller):
         if self._preferences is None:
-            self._preferences = Preferences(self._window, self._config)
+            self._preferences = Preferences(self._window, self._config, self._lastfm_connected)
             self._preferences.show_all()
     
         self._preferences.show()
 
-    def _destroy_banner(self, caller):
+    def _on_library_updated(self, caller, was_updated):
         gobject.idle_add(self._banner.destroy)
+        
+        if was_updated:
+            gobject.idle_add(self._refresh_playlist)
     
-    def _on_key_press(self, widget, event):
+    def _handle_keypress(self, widget, event):
         key = gtk.gdk.keyval_name(event.keyval)
         
         if key == "Right":
@@ -200,5 +248,11 @@ class Controller:
     def start(self):
         gtk.main()
         
-    def quit(self, caller=None):
+    def quit(self, caller=None):        
+        if self._lastfm_connected:
+            try:
+                scrobbler.flush()
+            except urllib2.URLError, e:
+                sys.stderr.write("lastfm.submit: %s\n" % e)
+        
         gtk.main_quit()
\ No newline at end of file
