@@ -122,8 +122,8 @@ class Library(GObject):
         act_files = self._get_files(folder)
         db_files = cursor.execute("SELECT uri FROM songs").fetchall()
         
-        updated = len(act_files) is not len(db_files)
-
+        updated = len(act_files) != len(db_files)
+        
         for uri, in db_files:
             try:
                 act_files.remove(str(uri))   # convert unicode
