@@ -2,11 +2,21 @@ import sys
 import gst
 import gobject
 import mutagen
+import time
 
 from gobject import GObject
 from youamp import Song, Playlist, MAX_VOL, IS_MAEMO
 
 class Player(GObject):
+    """
+    A Playlist based player
+    
+    signals:
+    song-changed: the song has changed
+    song-played: the song was played
+    tags-updated: the tags changed
+    seek-chaned: the seek changed
+    """
     __gsignals__ = {"song-changed": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                     "song-played": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                     "tags-updated": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
@@ -18,7 +28,8 @@ class Player(GObject):
         self.playlist = Playlist()
         self._current = None
         self._player = None
-        self._seek_emitting = False
+        self._seek_emit_id = None
+        self._set_duration_id = None
         self._config = config
         self._vol = config["volume"] * MAX_VOL
         
@@ -26,10 +37,7 @@ class Player(GObject):
         self._config.notify_add("rg-preamp", lambda client, cxn_id, entry, data: self._apply_volume())
         self._config.notify_add("no-rg-preamp", lambda client, cxn_id, entry, data: self._apply_volume())
 
-    def next(self, bus=None, message=None):
-        if self.get_time() > 30:
-            self.emit("song-played", self._current)
-        
+    def next(self, bus=None, message=None):        
         self._config["pos"] = (self._config["pos"] + 1) % len(self.playlist)        
         self.start_playback()
 
@@ -37,17 +45,12 @@ class Player(GObject):
         return len(self.playlist)
 
     def goto_pos(self, pos):
-        if self.get_time() > 30:
-            self.emit("song-played", self._current)
-
         self._config["pos"] = pos % len(self.playlist)        
         self.start_playback()
 
-    def previous(self):            
-        if self.get_time() > 30:
-            self.emit("song-played", self._current)
-
+    def previous(self):
         if self.get_time() > 3:
+            self._emit_played()
             self.seek_to(0.0)
             return
 
@@ -57,17 +60,17 @@ class Player(GObject):
     def get_time(self):
         """returns position in seconds"""
         try:
-            s = self._player.query_position(gst.FORMAT_TIME)[0]/ 1000000000 # ns*10^-9 = s
+            s = self._player.query_position(gst.FORMAT_TIME)[0]/ gst.SECOND
             return s
-        except gst.QueryError:
-            sys.stderr.write("get_time: player not ready\n")
+        except gst.QueryError, e:
+            sys.stderr.write("get_time: %s\n" % e)
 
     def get_seek(self):
         """returns position as fraction"""
         try:
             length = float(self._player.query_duration(gst.FORMAT_TIME)[0])
-        except gst.QueryError:
-            sys.stderr.write("get_seek: Query Error\n")
+        except gst.QueryError, e:
+            sys.stderr.write("get_seek: %s\n" % e)
             return 0.0
         else:            
             return self._player.query_position(gst.FORMAT_TIME)[0]/length
@@ -75,16 +78,23 @@ class Player(GObject):
     def seek_to(self, pos):
         try:
             length = float(self._player.query_duration(gst.FORMAT_TIME)[0])
-        except gst.QueryError:
-            sys.stderr.write("seek_to: Query Error\n")
+        except gst.QueryError, e:
+            sys.stderr.write("seek_to: %s\n" % e)
         else:        
             self._player.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH, int(length*pos))
-
+    
+    def _emit_played(self):
+        self.emit("song-played", self._current)
+            
     def load_track(self):
+        # emit song-played for previous track
+        if self._current is not None:
+            self._emit_played()
+
         try:
             self._current = self.playlist[self._config["pos"]]
-        except IndexError:
-            sys.stderr.write("Error: invalid playlist position\n")
+        except IndexError, e:
+            sys.stderr.write("load_track: %s\n" % e)
             return
         
         if self._player is not None:
@@ -100,17 +110,32 @@ class Player(GObject):
         bus.connect("message::tag", self._on_tag)
         bus.connect("message::eos", self.next)
         bus.connect("message::error", self._on_error)
+        bus.connect("message::state-changed", self._on_state_changed)
 
         self._player.set_property("uri", self._current.uri)
         self._player.set_state(gst.STATE_PAUSED)
-        # up to here core tag events were fired (rg info not there yet)
-                
-        self.emit("song-changed", self._current)
 
+    def _on_state_changed(self, bus, message):
+        # wait for playbin to move from ready to paused
+        # up to there all tag events were fired
+        if message.src.get_name().startswith("playbin"):
+            state = message.parse_state_changed()
+            if state[0:2] == [gst.STATE_READY, gst.STATE_PAUSED]:
+                self._apply_volume()
+                
+                # FIXME actually duration should be set here
+                if self._set_duration_id is not None:
+                    gobject.source_remove(self._set_duration_id)
+                self._set_duration_id = gobject.timeout_add(1000, self._set_duration)
+                self.emit("song-changed", self._current)
+    
+    def _set_duration(self):
+         self._current["duration"] = int(self._player.query_duration(gst.FORMAT_TIME)[0]/gst.SECOND)
+         return False
+   
     def start_playback(self):
         if not self.playing:
-            gobject.timeout_add(1000, self._emit_seek)
-            self._seek_emitting = True
+            self._emit_seek_id = gobject.timeout_add(1000, self._emit_seek)
 
         self.playing = True
         self.load_track()
@@ -118,14 +143,14 @@ class Player(GObject):
 
     def _on_error(self, bus, message):
         sys.stderr.write("Error: %s\n" % message.parse_error()[0])
-
+        
     def __del__(self):
         self._player.set_state(gst.STATE_NULL)
 
     def _on_tag(self, bus, message):
         self._update_song(message.parse_tag())
-        self._apply_volume()
         self.emit("tags-updated", self._current)
+        
     
     def mute(self):
         self._player.set_property("volume", 0.0)
@@ -141,13 +166,12 @@ class Player(GObject):
         if self.playing:
             self._player.set_state(gst.STATE_PAUSED)
             self.playing = False
+            gobject.source_remove(self._emit_seek_id)
         else:
             self._player.set_state(gst.STATE_PLAYING)
             self.playing = True
             
-            if not self._seek_emitting:
-                gobject.timeout_add(1000, self._emit_seek)
-                self._seek_emitting = True
+            self._emit_seek_id = gobject.timeout_add(1000, self._emit_seek)
 
     def _on_buffering(self, bus, message):
         print message.structure["buffer-percent"]
@@ -200,11 +224,6 @@ class Player(GObject):
 
     def _emit_seek(self):
         # do not send update if we are already paused
-        if self.playing:
-            self.emit("seek-changed", self.get_seek())
-        else:
-            self._seek_emitting = False
+        self.emit("seek-changed", self.get_seek())
         
-        print "foo"
-        
-        return self.playing # if True will be called repeatedly
+        return True # if True will be called repeatedly
