@@ -35,7 +35,7 @@ class AboutDialog(gtk.AboutDialog):
         self.set_authors(("Pavel Rojtberg",))
         self.set_artists(("Pavel Rojtberg", "Tango Icon Team"))
         self.set_translator_credits(_("translator-credits"))
-        self.set_comments("A Rhythmbox like Music Player for the Internet Tablet")
+        self.set_comments("A lightweight Music Player for the Internet Tablet")
         
         if IS_MAEMO:
             logo = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/youamp.png")
