@@ -43,7 +43,7 @@ class Window(hildon.Window):
         vbox.show()
         self.add(vbox)
     
-        self._player.connect("tags-updated", self._on_tags_updated)
+        self._player.connect("song-changed", self._update_songinfo)
         
         self.show()
     
@@ -57,7 +57,7 @@ class Window(hildon.Window):
         
         self._is_fullscreen = not self._is_fullscreen
     
-    def _on_tags_updated(self, caller, newsong):
+    def _update_songinfo(self, caller, newsong):
         self.set_title("%s - %s" % (newsong["title"], newsong["artist"]))
                 
         label_txt = "<b><big>%s</big></b> %s <i>%s</i> %s <i>%s</i>" % (newsong["title"], _("by"), newsong["artist"], _("from"), newsong["album"])
